@@ -1,4 +1,5 @@
 import logging
+import time
 
 try:
     from http.server import BaseHTTPRequestHandler, HTTPServer
@@ -16,6 +17,7 @@ import ssl
 
 from temboardagent.routing import get_routes
 from temboardagent.errors import HTTPError
+from temboardagent.tools import JSONEncoder
 from temboardagent import __version__ as temboard_version
 from .sharedmemory import Sessions
 from .api import check_sessionid
@@ -100,6 +102,26 @@ class RequestHandler(BaseHTTPRequestHandler):
         logger.info("client: %s request: %s"
                     % (self.address_string(), format % args))
 
+    def handle_one_request(self, *a, **kw):
+        self.start_time = time.time()
+        return BaseHTTPRequestHandler.handle_one_request(self, *a, **kw)
+
+    def request_time(self):
+        return 1000. * (time.time() - self.start_time)
+
+    def log_request(self, code='-', size='-'):
+        """Log an accepted request.
+
+        This is called by send_response().
+
+        """
+        if hasattr(code, 'value'):
+            code = code.value
+        self.log_message(
+            '"%s" %s %s %.2fms',
+            self.requestline, str(code), str(size), self.request_time(),
+        )
+
     def response(self):
         """
         In charge to call the main routing function and to return its results
@@ -131,7 +153,8 @@ class RequestHandler(BaseHTTPRequestHandler):
             self.send_header('Access-Control-Allow-Origin', '*')
             self.send_header('Content-type', 'application/json')
             self.end_headers()
-            self.wfile.write(json.dumps(message).encode('utf-8'))
+            self.wfile.write(json.dumps(message, cls=JSONEncoder)
+                             .encode('utf-8'))
         except Exception as e:
             logger.exception(str(e))
             logger.error("Could not send response")
