@@ -1,9 +1,11 @@
+import errno
+import hashlib
+import json
 import logging
 import os
-import hashlib
-import time
 import re
-import errno
+import time
+from datetime import datetime
 from time import strftime, gmtime
 from temboardagent.errors import HTTPError
 
@@ -132,7 +134,7 @@ def check_fqdn(name):
     """
     # StackOverflow #11809631
     if re.match(r'(?=^.{4,253}$)(^((?!-)[a-zA-Z0-9-]{1,63}(?<!-)\.)+[a-zA-Z]'  # noqa W605
-                '{2,63}\.?$)', name):
+                r'{2,63}\.?$)', name):
         return True
     else:
         return False
@@ -141,3 +143,11 @@ def check_fqdn(name):
 def now():
     """Give the current date and time at GMT, suitable for PostgreSQL."""
     return strftime("%Y-%m-%d %H:%M:%S +0000", gmtime())
+
+
+class JSONEncoder(json.JSONEncoder):
+    def default(self, obj):
+        if isinstance(obj, datetime):
+            return obj.isoformat()
+        else:
+            return super(JSONEncoder, self).default(obj)
